"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.middleware = middleware;
exports.runFetch = runFetch;
exports.runFetchAction = runFetchAction;
exports.runFetchMultiple = runFetchMultiple;
exports.default = exports.fetchError = exports.fetchMultiple = exports.fetchAction = exports.decrementFetches = exports.incrementFetches = exports.FETCH_ERROR = exports.FETCH_MULTIPLE = exports.FETCH = exports.DECREMENT_FETCH = exports.INCREMENT_FETCH = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _isObject = _interopRequireDefault(require("lodash/isObject"));

if (typeof fetch === 'undefined') {
  require('isomorphic-fetch');
}

var INCREMENT_FETCH = 'increment outstanding fetches';
exports.INCREMENT_FETCH = INCREMENT_FETCH;
var DECREMENT_FETCH = 'decrement outstanding fetches';
exports.DECREMENT_FETCH = DECREMENT_FETCH;
var FETCH = 'fetch';
exports.FETCH = FETCH;
var FETCH_MULTIPLE = 'fetch multiple';
exports.FETCH_MULTIPLE = FETCH_MULTIPLE;
var FETCH_ERROR = 'fetch error';
exports.FETCH_ERROR = FETCH_ERROR;

var createAction = function createAction(type) {
  return function (payload) {
    return {
      type: type,
      payload: payload
    };
  };
};

var incrementFetches = createAction(INCREMENT_FETCH);
exports.incrementFetches = incrementFetches;
var decrementFetches = createAction(DECREMENT_FETCH);
exports.decrementFetches = decrementFetches;
var fetchAction = createAction(FETCH);
exports.fetchAction = fetchAction;
var fetchMultiple = createAction(FETCH_MULTIPLE);
exports.fetchMultiple = fetchMultiple;
var fetchError = createAction(FETCH_ERROR);
exports.fetchError = fetchError;

function middleware(store) {
  return function (next) {
    return function (action) {
      if (action.type === FETCH) {
        return store.dispatch(runFetchAction(action.payload, store.getState()));
      } else if (action.type === FETCH_MULTIPLE) {
        return store.dispatch(runFetchMultiple(action.payload, store.getState()));
      } else {
        return next(action);
      }
    };
  };
}

var _default = fetchAction;
/**
 * Calls fetch, adds Auth and Content header if needed. Automatically parses
 * content based on type.
 *
 * @returns Promise
 */

exports.default = _default;

function runFetch(_ref, state) {
  var _ref$options = _ref.options,
      options = _ref$options === void 0 ? {} : _ref$options,
      _ref$retry = _ref.retry,
      retry = _ref$retry === void 0 ? false : _ref$retry,
      url = _ref.url;
  var headers = (0, _objectSpread2.default)({}, createAuthorizationHeader(state), createContentHeader(options.body), options.headers || {});
  var filteredHeaders = {}; // Allow removing generated headers by specifiying { header: null } in
  // options.headers. Do this in two steps because otherwise we're modifying
  // the object as we're iterating over it.

  Object.keys(headers).filter(function (key) {
    return headers[key] !== null && headers[key] !== undefined;
  }).forEach(function (key) {
    filteredHeaders[key] = headers[key];
  });
  return fetch(url, (0, _objectSpread2.default)({}, options, {
    body: serialize(options.body),
    headers: filteredHeaders
  })).then(checkStatus).then(createResponse).then(
  /*#__PURE__*/
  function () {
    var _ref2 = (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee(response) {
      return _regenerator.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.t0 = retry;

              if (!_context.t0) {
                _context.next = 5;
                break;
              }

              _context.next = 4;
              return retry(response);

            case 4:
              _context.t0 = _context.sent;

            case 5:
              if (!_context.t0) {
                _context.next = 9;
                break;
              }

              _context.t1 = runFetch({
                options: options,
                retry: retry,
                url: url
              }, state);
              _context.next = 10;
              break;

            case 9:
              _context.t1 = response;

            case 10:
              return _context.abrupt("return", _context.t1);

            case 11:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x) {
      return _ref2.apply(this, arguments);
    };
  }());
}

function runFetchAction(_ref3, state) {
  var next = _ref3.next,
      _ref3$options = _ref3.options,
      options = _ref3$options === void 0 ? {} : _ref3$options,
      _ref3$retry = _ref3.retry,
      retry = _ref3$retry === void 0 ? false : _ref3$retry,
      url = _ref3.url;
  var dispatchFetchError = !next || next.length < 2;
  var wrappedNext = wrapNext(next);
  return [incrementFetches({
    options: options,
    url: url
  }), runFetch({
    options: options,
    retry: retry,
    url: url
  }, state).then(function (response) {
    return [decrementFetches({
      options: options,
      url: url
    }), wrappedNext(null, response)];
  }).catch(function (error) {
    return createErrorResponse(error).then(function (response) {
      var actions = [decrementFetches({
        options: options,
        url: url
      }), wrappedNext(error, response)];
      if (dispatchFetchError) actions.push(fetchError(response));
      return actions;
    });
  })];
}
/**
 * @returns Promise
 */


function runFetchMultiple(_ref4, state) {
  var fetches = _ref4.fetches,
      next = _ref4.next;
  var dispatchFetchError = !next || next.length < 2;
  var wrappedNext = wrapNext(next);
  return (0, _toConsumableArray2.default)(fetches.map(function (_ref5) {
    var options = _ref5.options,
        url = _ref5.url;
    return incrementFetches({
      options: options,
      url: url
    });
  })).concat([Promise.all(fetches.map(function (fetch) {
    return runFetch(fetch, state);
  })).then(function (responses) {
    return (0, _toConsumableArray2.default)(fetches.map(function (_ref6) {
      var options = _ref6.options,
          url = _ref6.url;
      return decrementFetches({
        options: options,
        url: url
      });
    })).concat([wrappedNext(null, responses)]);
  }).catch(function (error) {
    return createErrorResponse(error).then(function (response) {
      var actions = fetches.map(function (_ref7) {
        var options = _ref7.options,
            url = _ref7.url;
        return decrementFetches({
          options: options,
          url: url
        });
      });
      if (dispatchFetchError) actions.push(fetchError(response));
      return (0, _toConsumableArray2.default)(actions).concat([wrappedNext(error, response)]);
    });
  })]);
}

function createAuthorizationHeader(state) {
  return state.user && state.user.idToken ? {
    Authorization: "bearer ".concat(state.user.idToken)
  } : {};
}

function checkStatus(res) {
  if (res.status >= 200 && res.status < 300) {
    return res;
  } else {
    throw res;
  }
}

function createContentHeader(body) {
  if (body instanceof window.FormData) {
    return {};
  } else if ((0, _isObject.default)(body)) {
    return {
      'Accept': 'application/json',
      'Content-Type': 'application/json;charset=UTF-8'
    };
  } else {
    return {};
  }
}

function createErrorResponse(res) {
  return res.headers ? createResponse(res) : Promise.resolve(res);
}

function createResponse(res) {
  return deserialize(res).then(function (value) {
    res.value = value;
    return res;
  }).catch(function (err) {
    res.value = err;
    return res;
  });
}

function deserialize(_x2) {
  return _deserialize.apply(this, arguments);
}

function _deserialize() {
  _deserialize = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee2(res) {
    var header;
    return _regenerator.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            header = "".concat(res.headers.get('Content-Type'), " ").concat(res.headers.get('Content'));

            if (!(header.indexOf('json') > -1)) {
              _context2.next = 3;
              break;
            }

            return _context2.abrupt("return", res.json());

          case 3:
            if (!(header.indexOf('octet-stream') > -1)) {
              _context2.next = 5;
              break;
            }

            return _context2.abrupt("return", res.arrayBuffer());

          case 5:
            if (!(header.indexOf('text') > -1)) {
              _context2.next = 7;
              break;
            }

            return _context2.abrupt("return", res.text());

          case 7:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));
  return _deserialize.apply(this, arguments);
}

function serialize(body) {
  if (body instanceof window.FormData) {
    return body;
  } else if ((0, _isObject.default)(body)) {
    return JSON.stringify(body);
  } else {
    return body;
  }
}

function wrapNext(next) {
  return function (error, response) {
    if (next) {
      if (next.length > 1) {
        return next(error, response);
      } else if (!error) {
        return next(response);
      }
    }
  };
}

//# sourceMappingURL=fetch.js